/*
 * Arrebol Consultancy copyright.
 * 
 * This code belongs to Arrebol Consultancy
 * its use, redistribution or modification are prohibited 
 * without written authorization from Arrebol Consultancy.
 */
package com.arrebol.taxiservicios.beans.admin;

import com.arrebol.taxiservicios.beans.Datatable;
import com.arrebol.taxiservicios.beans.TaxiGenericBean;
import com.arrebol.taxiservicios.controller.admin.BitacoraController;
import com.arrebol.taxiservicios.controller.admin.GenericValidationController;
import com.arrebol.taxiservicios.controller.admin.TiempoAireController;
import com.arrebol.taxiservicios.model.admin.Taxi;
import com.arrebol.taxiservicios.model.admin.TiempoAire;
import com.arrebol.taxiservicios.model.core.User;
import com.arrebol.taxiservicios.model.enums.BasicType;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.view.ViewScoped;
import javax.inject.Named;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.primefaces.event.ReorderEvent;
import org.primefaces.event.RowEditEvent;

/**
 *
 * @author Oscar Vargas
 */
@Named("tiempoAireBean")
@ViewScoped
public class TiempoAireBean extends TaxiGenericBean implements Serializable, Datatable {

    
    /**
     * Update expense company list between start date and end date.
     */
    public void updateList() {
        logger.info("updateList");
        try {
            if (getStartDate().after(getEndDate())) {
                showMessage(FacesMessage.SEVERITY_ERROR,
                        getBundlePropertyFile().getString("payment.out"),
                        getBundlePropertyFile().getString("start.date.after.error"));
            } else {
                setUpDates(false);
                setTiempo(
                        getController().findBetweenDates(
                                getStartDate(),
                                getEndDate(),
                                getLoggedUser().getLocation().getId())
                );
            }
        } catch (Exception e) {
            logger.error("updateList", e);
        }
    }

    @Override
    public void addRow() {
        logger.info("addRow");
        
        if (getGenericController().existClosingDayByCreatedOn(new Date(), getLoggedUser())) {
                showMessage(FacesMessage.SEVERITY_WARN,
                        "Error",
                        "No se puede hacer ninguna acción si existe un corte del día");
                return;
            }
            if(getGenericController().existStableSmallBoxByCurdate(getLoggedUser()))
            {
                showMessage(FacesMessage.SEVERITY_WARN,
                            "Dia cerrado",
                            "No se puede hacer ninguna acción porque ya existe un cuadre de caja chica del día");
                return;
            }
            if(getGenericController().existStableGeneralBoxByCurdate(getLoggedUser()))
            {
                showMessage(FacesMessage.SEVERITY_WARN,
                            "Dia cerrado",
                            "No se puede hacer ninguna acción porque ya existe un cuadre de caja general del día");
                return;
            }
        try {
            
            TiempoAire ec = new TiempoAire(
                    UUID.randomUUID().toString(),
                    getAmountMayoreo(),
                    getAmountMenudeo(),
                    getReason(),
                    new Taxi(idTaxi),
                    vigencia,
                    BasicType.ENEBLED,
                    new User(getLoggedUser().getId(), ""),
                    new Date(),
                    getLoggedUser().getLocation());

            if (getController().save(ec)) {
                setUpDates(false);
                setTiempo(
                        getController().findBetweenDates(
                                getStartDate(),
                                getEndDate(),
                                getLoggedUser().getLocation().getId())
                );

                setAmountMayoreo(null);
                setAmountMenudeo(null);
                setReason(null);

                showMessage(FacesMessage.SEVERITY_INFO,
                        "Tiempo aire",
                        "Se agregó correctamente");
            } else {
                showMessage(FacesMessage.SEVERITY_ERROR,
                        "Tiempo aire",
                        "No se pudo agregar el tiempo aire");
            }
        } catch (Exception e) {
            logger.error("addRow", e);
            showMessage(FacesMessage.SEVERITY_FATAL,
                    "Tiempo aire",
                    "Ocurrio un error");
        }
    }

    @Override
    public void deleteRow() {
        logger.info("deleteRow");
        try {

            if (getGenericController().existClosingDayByCreatedOn(new Date(), getLoggedUser())) {
                showMessage(FacesMessage.SEVERITY_WARN,
                        "Error",
                        "No se puede hacer ninguna acción si existe un corte del día");
                return;
            }
            if(getGenericController().existStableSmallBoxByCurdate(getLoggedUser()))
            {
                showMessage(FacesMessage.SEVERITY_WARN,
                            "Dia cerrado",
                            "No se puede hacer ninguna acción porque ya existe un cuadre de caja chica del día");
                return;
            }
            if(getGenericController().existStableGeneralBoxByCurdate(getLoggedUser()))
            {
                showMessage(FacesMessage.SEVERITY_WARN,
                            "Dia cerrado",
                            "No se puede hacer ninguna acción porque ya existe un cuadre de caja general del día");
                return;
            }
            if (getController().updateStatus(getTiempoToUpdate().getId(), BasicType.DISABLED, new User(getLoggedUser().getId(), ""))) {

                if (null == getCommentsBitacora() || getCommentsBitacora().trim().equals("")) {
                    setCommentsBitacora("El usuario " + getLoggedUser().getUserName() + " no agrego comentarios a la hora de borrar este tiempo aire");
                }

                String actionBitacora = "Eliminar tiempo aire";
                String descBitacora = "Se eliminó el tiempo aire con monto "
                        + currencyFormatNumber(getTiempoToUpdate().getAmountMayoreo()) + " y fecha " + getTiempoToUpdate().getCreatedOn();

                getBitacoraController().saveNewRecordInBitacora(createNewRecordInBitacora(UUID.randomUUID().toString(), actionBitacora, getCommentsBitacora(), descBitacora), 
                        getLoggedUser().getLocation().getId());
                getTiempo().remove(getTiempoToUpdate());

                setTiempoToUpdate(null);
                setCommentsBitacora("");
                showMessage(FacesMessage.SEVERITY_INFO,
                        "Tiempo aire",
                        "Se eliminó correctamente");
            } else {
                showMessage(FacesMessage.SEVERITY_ERROR,
                        "Tiempo aire",
                        "No se pudo eliminar");
            }
        } catch (Exception e) {
            logger.error("deleteRow: " + e);

            showMessage(FacesMessage.SEVERITY_FATAL,
                     "Tiempo aire",
                    "Ocurrió un error");
        }
    }

    @Override
    public void editRow(RowEditEvent event) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void onRowCancel(RowEditEvent event) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void onRowReorder(ReorderEvent event) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    /**
     * Setup start date and end date.
     *
     * @param init True if comes from @PostConstruct otherwise false.
     */
    private void setUpDates(boolean init) {
        Calendar startCalendar = Calendar.getInstance();
        Calendar endCalendar = Calendar.getInstance();

        if (!init) {
            startCalendar.setTime(getStartDate());
            endCalendar.setTime(getEndDate());
        } else {
            startCalendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
            endCalendar.setTime(new Date());
        }

        startCalendar.add(Calendar.HOUR_OF_DAY, 0);
        startCalendar.add(Calendar.MINUTE, 0);
        startCalendar.add(Calendar.SECOND, 0);
        startCalendar.add(Calendar.MILLISECOND, 0);

        endCalendar.add(Calendar.HOUR_OF_DAY, 22);
        endCalendar.add(Calendar.MINUTE, 59);
        endCalendar.add(Calendar.SECOND, 59);
        endCalendar.add(Calendar.MILLISECOND, 0);

        setStartDate(startCalendar.getTime());
        setEndDate(endCalendar.getTime());
    }

    @PostConstruct
    public void init() {
        logger.info("init");
        try {
            loadBundlePropertyFile();

            setUpDates(true);
            setController(new TiempoAireController());
            setBitacoraController(new BitacoraController());
            setGenericController(new GenericValidationController());
            setLastStableGeneralBox(getGenericController().lastStableGeneralBoxByDate(getLoggedUser()));
            setHayCuadre(getGenericController().existStableGeneralBoxByCurdate(getLoggedUser()));
            setLastStableSmallBox(getGenericController().lastStableSmallBoxByDate(getLoggedUser()));
            setHayCuadreSmall(getGenericController().existStableSmallBoxByCurdate(getLoggedUser()));
            
            setTiempo(
                    getController().findBetweenDates(
                            getStartDate(),
                            getEndDate(),
                            getLoggedUser().getLocation().getId())
            );
        } catch (Exception e) {
            logger.error(e);
        }
    }

    private TiempoAireController controller;
    private GenericValidationController genericController;
    private Date lastStableGeneralBox;
    private boolean hayCuadre;
    private Date lastStableSmallBox;
    private boolean hayCuadreSmall;
    private List<TiempoAire> tiempo;
    private TiempoAire tiempoToUpdate;
    private Date startDate;
    private Date endDate;
    private BigDecimal amountMayoreo;
    private BigDecimal amountMenudeo;
    private Date vigencia;
    private String reason;
    private BitacoraController bitacoraController;
    private String commentsBitacora;
    private String idTaxi;

    public Date getLastStableSmallBox() {
        return lastStableSmallBox;
    }

    public void setLastStableSmallBox(Date lastStableSmallBox) {
        this.lastStableSmallBox = lastStableSmallBox;
    }

    public boolean isHayCuadreSmall() {
        return hayCuadreSmall;
    }

    public void setHayCuadreSmall(boolean hayCuadreSmall) {
        this.hayCuadreSmall = hayCuadreSmall;
    }

    public boolean isHayCuadre() {
        return hayCuadre;
    }

    public void setHayCuadre(boolean hayCuadre) {
        this.hayCuadre = hayCuadre;
    }

    public GenericValidationController getGenericController() {
        return genericController;
    }

    public void setGenericController(GenericValidationController genericController) {
        this.genericController = genericController;
    }

    public Date getLastStableGeneralBox() {
        return lastStableGeneralBox;
    }

    public void setLastStableGeneralBox(Date lastStableGeneralBox) {
        this.lastStableGeneralBox = lastStableGeneralBox;
    }

    public TiempoAireController getController() {
        return controller;
    }

    public void setController(TiempoAireController controller) {
        this.controller = controller;
    }

    public List<TiempoAire> getTiempo() {
        return tiempo;
    }

    public void setTiempo(List<TiempoAire> tiempo) {
        this.tiempo = tiempo;
    }

    public TiempoAire getTiempoToUpdate() {
        return tiempoToUpdate;
    }

    public void setTiempoToUpdate(TiempoAire tiempoToUpdate) {
        this.tiempoToUpdate = tiempoToUpdate;
    }

    
    public Date getStartDate() {
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public String getReason() {
        return reason;
    }

    public void setReason(String reason) {
        this.reason = reason;
    }

    public BitacoraController getBitacoraController() {
        return bitacoraController;
    }

    public void setBitacoraController(BitacoraController bitacoraController) {
        this.bitacoraController = bitacoraController;
    }

    public String getCommentsBitacora() {
        return commentsBitacora;
    }

    public void setCommentsBitacora(String commentsBitacora) {
        this.commentsBitacora = commentsBitacora;
    }

    public BigDecimal getAmountMayoreo() {
        return amountMayoreo;
    }

    public void setAmountMayoreo(BigDecimal amountMayoreo) {
        this.amountMayoreo = amountMayoreo;
    }

    public BigDecimal getAmountMenudeo() {
        return amountMenudeo;
    }

    public void setAmountMenudeo(BigDecimal amountMenudeo) {
        this.amountMenudeo = amountMenudeo;
    }

    public Date getVigencia() {
        return vigencia;
    }

    public void setVigencia(Date vigencia) {
        this.vigencia = vigencia;
    }

    public String getIdTaxi() {
        return idTaxi;
    }

    public void setIdTaxi(String idTaxi) {
        this.idTaxi = idTaxi;
    }

    private static final long serialVersionUID = -136031528440947006L;
    final Logger logger = LogManager.getLogger(TiempoAireBean.class);
}
