/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.arrebol.taxiservicios.beans.payroll;

import com.arrebol.taxiservicios.beans.Datatable;
import com.arrebol.taxiservicios.beans.TaxiGenericBean;
import com.arrebol.taxiservicios.controller.admin.GenericValidationController;
import com.arrebol.taxiservicios.controller.payroll.AdelantosController;
import com.arrebol.taxiservicios.model.core.User;
import com.arrebol.taxiservicios.model.payroll.Adelantos;
import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.view.ViewScoped;
import javax.inject.Named;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.primefaces.event.ReorderEvent;
import org.primefaces.event.RowEditEvent;

/**
 *
 * @author Oscar
 */
@Named
@ViewScoped
public class AdelantosBean extends TaxiGenericBean implements Serializable, Datatable{
    
    public void updateHistoryAdelantoList() {
        logger.info("updateHistoryAdelantoList");
        try {
            if (getStartDate().after(getEndDate())) {
                showMessage(FacesMessage.SEVERITY_ERROR,
                        getBundlePropertyFile().getString("stable.general.box"),
                        getBundlePropertyFile().getString("start.date.after.error"));
            } else {
                setUpDates(false);
                setAdelantos(
                        getController().findAdelantosBetweenDates(
                                getStartDate(),
                                getEndDate(),
                                getLoggedUser().getLocation().getId()
                        )
                );
            }
        } catch (Exception e) {
            logger.error("updateHistoryAdelantoList", e);
        }
    }
    
    @Override
    public void deleteRow() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
    @Override
    public void editRow(RowEditEvent event) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void onRowCancel(RowEditEvent event) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void onRowReorder(ReorderEvent event) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void addRow() {
        if(getGenericController().existStableGeneralBoxByCurdate(getLoggedUser()))
            {
                showMessage(FacesMessage.SEVERITY_WARN,
                            "Dia cerrado",
                            "No se pueden generar más adelantos porque ya existe un cuadre de caja general del día");
                return;
            }
        getAddAdelanto().setId(UUID.randomUUID().toString());
        getAddAdelanto().setActive(true);
        getAddAdelanto().setEmployee(getController().findUserById(idUser).getPerson());
        getAddAdelanto().setCreatedBy(getLoggedUser());
        getAddAdelanto().setCreatedOn(new Date());
        getAddAdelanto().setLocation(getLoggedUser().getLocation());
        
        if (getController().save(getAddAdelanto())) {

                setUpDates(false);
                setAdelantos(
                        getController().findAdelantosBetweenDates(
                                getStartDate(),
                                getEndDate(),
                                getLoggedUser().getLocation().getId()
                        )
                );
                
                showMessage(FacesMessage.SEVERITY_INFO,
                                "Adelanto de nómina",
                                "Se agregó correctamente");
                
                initAdelantosObject();
        } else {
                    logger.error("adelanto couldnot saved");
                    showMessage(FacesMessage.SEVERITY_ERROR,
                            "Adelanto de nómina",
                            "No se puede crear el adelanto");
                }
    }
    
    @PostConstruct
    public void init() {
        try {
            loadBundlePropertyFile();
            setController(new AdelantosController());
            setCreatedOn(new Date());
            setUpDates(true);
            
            setGenericController(new GenericValidationController());
            setLastStableGeneralBox(getGenericController().lastStableGeneralBoxByDate(getLoggedUser()));
            setHayCuadre(getGenericController().existStableGeneralBoxByCurdate(getLoggedUser()));

            setAdelantos(
                    getController().findAdelantosBetweenDates(
                            getStartDate(),
                            getEndDate(),
                            getLoggedUser().getLocation().getId()
                    )
            );
            
            setUsers(getController().fillUserByLocation(getLoggedUser().getLocation()));
            
            initAdelantosObject();
            
        } catch (Exception e) {
            logger.error(e);
        }

    }
    
    private void initAdelantosObject() {
        logger.info("initPayrollObject");
        try {
            setAddAdelanto(new Adelantos());

            getAddAdelanto().setAmount(0D);
            getAddAdelanto().setComment("");
        } catch (Exception e) {
            logger.error("initAdelantosObject", e);
        }

    }
    
    private void setUpDates(boolean init) {
        Calendar startCalendar = Calendar.getInstance();
        Calendar endCalendar = Calendar.getInstance();

        if (!init) {
            startCalendar.setTime(getStartDate());
            endCalendar.setTime(getEndDate());
        } else {
            startCalendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
            endCalendar.setTime(new Date());
        }

        startCalendar.add(Calendar.HOUR_OF_DAY, 2);
        startCalendar.add(Calendar.MINUTE, 0);
        startCalendar.add(Calendar.SECOND, 0);
        startCalendar.add(Calendar.MILLISECOND, 0);

        endCalendar.add(Calendar.HOUR_OF_DAY, 22);
        endCalendar.add(Calendar.MINUTE, 59);
        endCalendar.add(Calendar.SECOND, 59);
        endCalendar.add(Calendar.MILLISECOND, 0);

        setStartDate(startCalendar.getTime());
        setEndDate(endCalendar.getTime());
    }
    
    private GenericValidationController genericController;
    private Date lastStableGeneralBox;
    private boolean hayCuadre;
    private AdelantosController controller;
    private List<Adelantos> adelantos;
    private List<User> users;
    private Adelantos addAdelanto;
    private Adelantos selectedAdelanto;
    private Date startDate;
    private Date endDate;
    private Date createdOn;
    private String idUser;

    public GenericValidationController getGenericController() {
        return genericController;
    }

    public void setGenericController(GenericValidationController genericController) {
        this.genericController = genericController;
    }

    public Date getLastStableGeneralBox() {
        return lastStableGeneralBox;
    }

    public void setLastStableGeneralBox(Date lastStableGeneralBox) {
        this.lastStableGeneralBox = lastStableGeneralBox;
    }

    public boolean isHayCuadre() {
        return hayCuadre;
    }

    public void setHayCuadre(boolean hayCuadre) {
        this.hayCuadre = hayCuadre;
    }

    public AdelantosController getController() {
        return controller;
    }

    public void setController(AdelantosController controller) {
        this.controller = controller;
    }

    public List<Adelantos> getAdelantos() {
        return adelantos;
    }

    public void setAdelantos(List<Adelantos> adelantos) {
        this.adelantos = adelantos;
    }

    public List<User> getUsers() {
        return users;
    }

    public void setUsers(List<User> users) {
        this.users = users;
    }

    public Adelantos getAddAdelanto() {
        return addAdelanto;
    }

    public void setAddAdelanto(Adelantos addAdelanto) {
        this.addAdelanto = addAdelanto;
    }

    public Adelantos getSelectedAdelanto() {
        return selectedAdelanto;
    }

    public void setSelectedAdelanto(Adelantos selectedAdelanto) {
        this.selectedAdelanto = selectedAdelanto;
    }

    public Date getStartDate() {
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public Date getCreatedOn() {
        return createdOn;
    }

    public void setCreatedOn(Date createdOn) {
        this.createdOn = createdOn;
    }

    public String getIdUser() {
        return idUser;
    }

    public void setIdUser(String idUser) {
        this.idUser = idUser;
    }
    
    private static final long serialVersionUID = 4735474224887176743L;
    final Logger logger = LogManager.getLogger(AdelantosBean.class);
}
